/* Cubic Resampler v0.1
 * By DOLLS
 * Do not distribute a modified version without permission. 
 */

texture tex1, tex2;
float2 rcpres;
sampler s0 = sampler_state {
	texture = <tex1>;
	AddressU = mirror;
	AddressV = mirror;
};

sampler s1 = sampler_state {
	texture = <tex2>;
	AddressU = mirror;
	AddressV = mirror;
};

int2 res;

float4 Normalize(in float2 coord : TEXCOORD0): COLOR0
{		
	res = int2(256, (frac(1 / (224 * rcpres.y)) < 0.05)? 224 : 239);
	float2 loc = coord * float2(rcpres.y * res.y / (rcpres.x * res.x), 1.0) / (res.y * rcpres.y);
	return tex2D(s0, loc);
}

/*
 * The following parameters adjust the weights for the cubic spline.
 * They allow for different appearances in the interpolated image.
 *
 * Examples:
 *
 * Defined Samples:		B = 0.0, C = 0.0
 * Gives a pixelated look.
 *
 * Catmull-Rom Spline:	B = 0.0, C = 0.5
 * Approximates a 2-lobe windowed sinc, sharp results.
 *
 * Standard Cubic:		B = 1.0, C = 0.0
 * Smooth appearance.
 */
 
#define	B	0.0
#define	C	0.5

/*
 * Do NOT modify the following scalars.
 */
#define	P0	(6.0 - 2.0 * B) / 6.0
#define	P2	(-18.0 + 12.0 * B + 6.0 * C) / 6.0
#define	P3	(12.0 - 9.0 * B - 6.0 * C) / 6.0
#define	Q0	(8.0 * B + 24.0 * C) / 6.0
#define	Q1	(-12.0 * B - 48.0 * C) / 6.0
#define	Q2	(6.0 * B + 30.0 * C) / 6.0
#define	Q3	(-B - 6.0 * C) / 6.0
#define	Q3	(-B - 6.0 * C) / 6.0

float getWeight(float x)
{	
	if (x < 1.0) return P0 + x * x * (P2 + x * P3);
	else if (x < 2.0) return Q0 + x * (Q1 + x * (Q2 + x * Q3));
	else return 0.0;  		
}

float4 Resample(in float2 coord : TEXCOORD0) : COLOR0
{	
	res = int2(256, (frac(1 / (224 * rcpres.y)) < 0.05)? 224 : 239);
	float2 pos = coord * res - 0.5;
	float2 tex = (floor(pos) + 0.5) * rcpres;
	
	float2 delta = frac(pos);	
	float weight[4];
	float contrib = 0.0;
	
	for (int i = -1; i <= 2; i++)
		contrib += weight[i + 1] = getWeight(abs(i - delta.y));
		
	float4 value = float4(0.0, 0.0, 0.0, 0.0);
	for (int i = -1; i <= 2; i++) {                       
		float ele = getWeight(abs(delta.x - i));		
		float j = i * rcpres.x;
		value += ele * weight[0] * tex2D(s1, tex + float2(j, -rcpres.y));
		value += ele * weight[1] * tex2D(s1, tex + float2(j, 0.0));
		value += ele * weight[2] * tex2D(s1, tex + float2(j, rcpres.y));
		value += ele * weight[3] * tex2D(s1, tex + float2(j, rcpres.y * 2.0));
	}		
	return value / (contrib * contrib);
}

Technique T0
{		
	pass p0 {		
		PixelShader = compile ps_2_0 Normalize();
	}	
	pass p1 {
		PixelShader = compile ps_2_a Resample();
	}			
} 